/*!
 *****************************************************************************
 * @file:    startup.c
 * @brief:   Interrupt table and default handlers for ADSP-CM40z
 * @version: $Revision: 31405 $
 * @date:    $Date: 2015-06-11 12:30:30 -0400 (Thu, 11 Jun 2015) $
 *-----------------------------------------------------------------------------
 *
Copyright (c) 2010 Analog Devices, Inc.

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  - Modified versions of the software must be conspicuously marked as such.
  - This software is licensed solely and exclusively for use with processors
    manufactured by or for Analog Devices, Inc.
  - This software may not be combined or merged with other code in any manner
    that would cause the software to become subject to terms and conditions
    which differ from those listed here.
  - Neither the name of Analog Devices, Inc. nor the names of its
    contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.
  - The use of this software may or may not infringe the patent rights of one
    or more patent holders.  This license does not release you from the
    requirement that you obtain separate licenses from these patent holders
    to use this software.

THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES, INC. AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
TITLE, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
NO EVENT SHALL ANALOG DEVICES, INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, DAMAGES ARISING OUT OF CLAIMS OF INTELLECTUAL
PROPERTY RIGHTS INFRINGEMENT; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#include "startup.h"

#ifdef ADI_MT_SUPPORT
/*
 * Micrium uCOS-III redefines these handler names.  If the application
 * uses uCOS-III the macro ADI_MT_SUPPORT must be added to the project
 * preprocessor options.
 */
#define PendSV_Handler  OS_CPU_PendSVHandler
#define SysTick_Handler OS_CPU_SysTickHandler
#endif

/*
    Stack size considerations for RTOS environments:

    In the Cortex-M class processors, the hardware has two stacks: the main stack
    and the process stack.  The main stack is used at reset and by system exception
    handlers.  In simple applications (non-RTOS) you may only use this stack.  In
    more complex systems that use an RTOS you have a split between the main stack
    which is used by the kernel and exception handlers and the process stack which
    is used by the tasks and programmable interrupts (interrupts stack on top of
    the currently active task, so tasks stacks should be allocated for worst-case
    interrupt scenarios).  In the uCOS case, the process stack is the one that gets
    context-switched and used by the tasks, where we store the registers, etc.

    As RTOS tasks specify their own stack area, a minimal size of 4 for the process
    stack in order to retain symbols, etc., seems OK and we would not need any more.
    The main stack needs to be big enough to initialize the RTOS, run interrupts etc.

    As always, stack and heap size optimizations is a balancing act between usage
    and available memory.  It is entirely application-dependant.

*/


//*****************************************************************************
//
// Local helpers for consistent ISR prototypes and implementations.
//
//*****************************************************************************
#define WEAK_PROTOTYPE(x) WEAK_PROTO ( void x (void)) ATTRIBUTE_INTERRUPT ;

#ifdef HANDLERS_IN_FLASH
// Locate the handlers in flash. The default linker .icf file copies "ro" code
// to RAM. To place the handlers in flash the .icf file must be modified to
// not copy "ro" code:
//   initialize by copy { rw };
#define WEAK_FUNCTION(x)  WEAK_FUNC  ( void x (void)) { while(1); }
#else
// Note: because the reset handler (ResetISR) is where the startup lives,
// the linker can't to initialize it because its needed for startup.  That
// causes the linker to mark the entire module as un-initializable and would
// result in all handlers being located in flash (along with the reset vector).
// Marking the bulk of the interrupt handlers as "__ramfunc" tells the linker
// its OK to locate them in RAM as part of the initialization process.  It
// puts them into a special ".textrw" section which is placed explicitly
// into the RAM_CODE region by the icf file (in addition to ".text").
#define WEAK_FUNCTION(x)  WEAK_FUNC  ( __ramfunc void x (void)) { while(1); }
#endif

//*****************************************************************************
//
// Forward declaration of the system exceptions.
//
//*****************************************************************************
WEAK_PROTOTYPE( ResetISR                           )
WEAK_PROTOTYPE( NmiSR                              )
WEAK_PROTOTYPE( HardFault_Handler                  )
WEAK_PROTOTYPE( MemManage_Handler                  )
WEAK_PROTOTYPE( BusFault_Handler                   )
WEAK_PROTOTYPE( UsageFault_Handler                 )
WEAK_PROTOTYPE( SVC_Handler                        )
WEAK_PROTOTYPE( DebugMon_Handler                   )
WEAK_PROTOTYPE( PendSV_Handler                     )
WEAK_PROTOTYPE( SysTick_Handler                    )

//*****************************************************************************
//
// Forward declaration of the programmable device interrupt handlers.
//
//*****************************************************************************
WEAK_PROTOTYPE( OSCW_EVT_Int_Handler               )
WEAK_PROTOTYPE( CGU0_EVT_Int_Handler               )
WEAK_PROTOTYPE( M4P_L1CC_PERR_Int_Handler          )
WEAK_PROTOTYPE( M4P_CORE_SRAM_PERR_Int_Handler     )
WEAK_PROTOTYPE( M4P_DMA_SRAM_PERR_Int_Handler      )
WEAK_PROTOTYPE( M4P_BUS_FAULT_Int_Handler          )
WEAK_PROTOTYPE( M4P_LOCKUP_Int_Handler             )
WEAK_PROTOTYPE( M4P_SRAM_PERR_FLT_Int_Handler      )
WEAK_PROTOTYPE( WDOG0_EXP_Int_Handler              )
WEAK_PROTOTYPE( SEC0_ERR_Int_Handler               )
WEAK_PROTOTYPE( ECT_EVT0_Int_Handler               )
WEAK_PROTOTYPE( ECT_EVT1_Int_Handler               )
WEAK_PROTOTYPE( PWM0_TRIP_Int_Handler              )
WEAK_PROTOTYPE( PWM1_TRIP_Int_Handler              )
WEAK_PROTOTYPE( PWM2_TRIP_Int_Handler              )
WEAK_PROTOTYPE( PWM0_SYNC_Int_Handler              )
WEAK_PROTOTYPE( PWM1_SYNC_Int_Handler              )
WEAK_PROTOTYPE( PWM2_SYNC_Int_Handler              )
WEAK_PROTOTYPE( PINT0_BLOCK_Int_Handler            )
WEAK_PROTOTYPE( PINT1_BLOCK_Int_Handler            )
WEAK_PROTOTYPE( PINT2_BLOCK_Int_Handler            )
WEAK_PROTOTYPE( PINT3_BLOCK_Int_Handler            )
WEAK_PROTOTYPE( PINT4_BLOCK_Int_Handler            )
WEAK_PROTOTYPE( SPORT0_A_DMA_ERR_Int_Handler       )
WEAK_PROTOTYPE( SPORT0_B_DMA_ERR_Int_Handler       )
WEAK_PROTOTYPE( SPI0_TXDMA_ERR_Int_Handler         )
WEAK_PROTOTYPE( SPI0_RXDMA_ERR_Int_Handler         )
WEAK_PROTOTYPE( UART0_TXDMA_ERR_Int_Handler        )
WEAK_PROTOTYPE( UART0_RXDMA_ERR_Int_Handler        )
WEAK_PROTOTYPE( SPORT1_A_DMA_ERR_Int_Handler       )
WEAK_PROTOTYPE( SPORT1_B_DMA_ERR_Int_Handler       )
WEAK_PROTOTYPE( SPI1_TXDMA_ERR_Int_Handler         )
WEAK_PROTOTYPE( SPI1_RXDMA_ERR_Int_Handler         )
WEAK_PROTOTYPE( UART1_TXDMA_ERR_Int_Handler        )
WEAK_PROTOTYPE( UART1_RXDMA_ERR_Int_Handler        )
WEAK_PROTOTYPE( UART2_TXDMA_ERR_Int_Handler        )
WEAK_PROTOTYPE( UART2_RXDMA_ERR_Int_Handler        )
WEAK_PROTOTYPE( HAE0_RXDMA_CH0_ERR_Int_Handler     )
WEAK_PROTOTYPE( HAE0_RXDMA_CH1_ERR_Int_Handler     )
WEAK_PROTOTYPE( HAE0_TXDMA_ERR_Int_Handler         )
WEAK_PROTOTYPE( SPI0_ERR_Int_Handler               )
WEAK_PROTOTYPE( SPI1_ERR_Int_Handler               )
WEAK_PROTOTYPE( SPI2_ERR_Int_Handler               )
WEAK_PROTOTYPE( ADCC0_ERR_Int_Handler              )
WEAK_PROTOTYPE( DACC0_ERR_Int_Handler              )
WEAK_PROTOTYPE( MDMA0_SRC_CRC0_IN_ERR_Int_Handler  )
WEAK_PROTOTYPE( MDMA0_DST_CRC0_OUT_ERR_Int_Handler )
WEAK_PROTOTYPE( MDMA1_SRC_ERR_Int_Handler          )
WEAK_PROTOTYPE( MDMA1_DST_ERR_Int_Handler          )
WEAK_PROTOTYPE( TIMER0_STAT_Int_Handler            )
WEAK_PROTOTYPE( TIMER0_TMR0_Int_Handler            )
WEAK_PROTOTYPE( TIMER0_TMR1_Int_Handler            )
WEAK_PROTOTYPE( TIMER0_TMR2_Int_Handler            )
WEAK_PROTOTYPE( TIMER0_TMR3_Int_Handler            )
WEAK_PROTOTYPE( TIMER0_TMR4_Int_Handler            )
WEAK_PROTOTYPE( TIMER0_TMR5_Int_Handler            )
WEAK_PROTOTYPE( TIMER0_TMR6_Int_Handler            )
WEAK_PROTOTYPE( TIMER0_TMR7_Int_Handler            )
WEAK_PROTOTYPE( CNT0_STAT_Int_Handler              )
WEAK_PROTOTYPE( CNT1_STAT_Int_Handler              )
WEAK_PROTOTYPE( CNT2_STAT_Int_Handler              )
WEAK_PROTOTYPE( CNT3_STAT_Int_Handler              )
WEAK_PROTOTYPE( SPORT0_A_STAT_Int_Handler          )
WEAK_PROTOTYPE( SPORT0_A_DMA_Int_Handler           )
WEAK_PROTOTYPE( SPORT0_B_STAT_Int_Handler          )
WEAK_PROTOTYPE( SPORT0_B_DMA_Int_Handler           )
WEAK_PROTOTYPE( UART0_STAT_Int_Handler             )
WEAK_PROTOTYPE( UART0_TXDMA_Int_Handler            )
WEAK_PROTOTYPE( UART0_RXDMA_Int_Handler            )
WEAK_PROTOTYPE( SINC0_STAT_Int_Handler             )
WEAK_PROTOTYPE( SPI0_STAT_Int_Handler              )
WEAK_PROTOTYPE( SPI0_TXDMA_Int_Handler             )
WEAK_PROTOTYPE( SPI0_RXDMA_Int_Handler             )
WEAK_PROTOTYPE( SPORT1_A_STAT_Int_Handler          )
WEAK_PROTOTYPE( SPORT1_A_DMA_Int_Handler           )
WEAK_PROTOTYPE( SPORT1_B_STAT_Int_Handler          )
WEAK_PROTOTYPE( SPORT1_B_DMA_Int_Handler           )
WEAK_PROTOTYPE( UART1_STAT_Int_Handler             )
WEAK_PROTOTYPE( UART1_TXDMA_Int_Handler            )
WEAK_PROTOTYPE( UART1_RXDMA_Int_Handler            )
WEAK_PROTOTYPE( SPI1_STAT_Int_Handler              )
WEAK_PROTOTYPE( SPI1_TXDMA_Int_Handler             )
WEAK_PROTOTYPE( SPI1_RXDMA_Int_Handler             )
WEAK_PROTOTYPE( SPI2_TX_Int_Handler                )
WEAK_PROTOTYPE( SPI2_RX_Int_Handler                )
WEAK_PROTOTYPE( EMAC0_STAT_Int_Handler             )
WEAK_PROTOTYPE( UART2_STAT_Int_Handler             )
WEAK_PROTOTYPE( UART2_TXDMA_Int_Handler            )
WEAK_PROTOTYPE( UART2_RXDMA_Int_Handler            )
WEAK_PROTOTYPE( HAE0_STAT_Int_Handler              )
WEAK_PROTOTYPE( HAE0_RXDMA_CH0_Int_Handler         )
WEAK_PROTOTYPE( HAE0_RXDMA_CH1_Int_Handler         )
WEAK_PROTOTYPE( HAE0_TXDMA_Int_Handler             )
WEAK_PROTOTYPE( ADCC0_TMR0_EVT_Int_Handler         )
WEAK_PROTOTYPE( ADCC0_TMR1_EVT_Int_Handler         )
WEAK_PROTOTYPE( DACC0_DAC0_Int_Handler             )
WEAK_PROTOTYPE( DACC0_DAC1_Int_Handler             )
WEAK_PROTOTYPE( SPI2_STAT_Int_Handler              )
WEAK_PROTOTYPE( TWI0_DATA_Int_Handler              )
WEAK_PROTOTYPE( CRC0_DCNTEXP_Int_Handler           )
WEAK_PROTOTYPE( CRC0_ERR_Int_Handler               )
WEAK_PROTOTYPE( MDMA0_SRC_CRC0_IN_Int_Handler      )
WEAK_PROTOTYPE( MDMA0_DST_CRC0_OUT_Int_Handler     )
WEAK_PROTOTYPE( MDMA1_SRC_Int_Handler              )
WEAK_PROTOTYPE( MDMA1_DST_Int_Handler              )
WEAK_PROTOTYPE( USB0_STAT_Int_Handler              )
WEAK_PROTOTYPE( USB0_DATA_Int_Handler              )
WEAK_PROTOTYPE( TRU0_INT0_Int_Handler              )
WEAK_PROTOTYPE( TRU0_INT1_Int_Handler              )
WEAK_PROTOTYPE( TRU0_INT2_Int_Handler              )
WEAK_PROTOTYPE( TRU0_INT3_Int_Handler              )
WEAK_PROTOTYPE( CGU0_ERR_Int_Handler               )
WEAK_PROTOTYPE( DPM0_EVT_Int_Handler               )
WEAK_PROTOTYPE( SOFT0_Int_Handler                  )
WEAK_PROTOTYPE( SOFT1_Int_Handler                  )
WEAK_PROTOTYPE( SOFT2_Int_Handler                  )
WEAK_PROTOTYPE( SOFT3_Int_Handler                  )
WEAK_PROTOTYPE( SWU0_EVT_Int_Handler               )
WEAK_PROTOTYPE( SWU1_EVT_Int_Handler               )
WEAK_PROTOTYPE( SWU2_EVT_Int_Handler               )
WEAK_PROTOTYPE( SWU3_EVT_Int_Handler               )
WEAK_PROTOTYPE( SWU4_EVT_Int_Handler               )
WEAK_PROTOTYPE( CAN0_RX_Int_Handler                )
WEAK_PROTOTYPE( CAN0_TX_Int_Handler                )
WEAK_PROTOTYPE( CAN0_STAT_Int_Handler              )
WEAK_PROTOTYPE( CAN1_RX_Int_Handler                )
WEAK_PROTOTYPE( CAN1_TX_Int_Handler                )
WEAK_PROTOTYPE( CAN1_STAT_Int_Handler              )
WEAK_PROTOTYPE( CPTMR0_CPT0_ERR_Int_Handler        )
WEAK_PROTOTYPE( CPTMR0_CPT1_ERR_Int_Handler        )
WEAK_PROTOTYPE( CPTMR0_CPT2_ERR_Int_Handler        )
WEAK_PROTOTYPE( CPTMR0_CPT0_MEAS_Int_Handler       )
WEAK_PROTOTYPE( CPTMR0_CPT1_MEAS_Int_Handler       )
WEAK_PROTOTYPE( CPTMR0_CPT2_MEAS_Int_Handler       )
WEAK_PROTOTYPE( USER_KEY_FAIL_Int_Handler          )
WEAK_PROTOTYPE( RESERVED_VECTOR                    )


/*******************************MANAGE SECURITY PROTOCOL********************************/
/* Generate code for security protocol by default for all ADSP-CM40z Silicon >= 2.0    */
/* For early silicon (<2.0), and for SRAM-based projects (that are using the "No-Boot" */
/* mode, assert the ADI_NO_SECURITY_PROTOCOL macro in each project's compiler          */
/* proprocessor defines section to generate startup code without security protocol,    */
/* i.e., only the native IVT at start of flash (or SRAM for RAM-based applications);   */
/* with NO SecurityHeader.                                                             */
/***************************************************************************************/


#ifndef ADI_NO_SECURITY_PROTOCOL
/*
   In secure mode, the Security Header resides at start of flash (0x18000000),
   contrary to ARM Cortex convention which usually requires the IVT to be placed
   at the start of flash.

   With the security header now occupying start of flash, the IVT is displaced,
   minding the next-higher-power-of-two alignment requirement on the IVT, which
   would normally only be 512 for this part.  However, the boot ROM expects the
   IVT to be moved down by 4k (the minimum erasable size within the flash), by
   default.  So we position the IVT at machine address 0x18001000 and the linker
   control file insures this placement.  Note that the IVT location is also
   encoded within the security header itself, in case it is moved by other than
   the expected default.

   The security header contains signature keys, user keys, boot flags, the IVT
   offset address, a reserved field and two computed ECC values.  The ECC fields
   contain Hamming (72,64) codes for security header error detection & correction
   by the boot ROM at system startup.  The EccGen utility is provided which parses
   the adi_ecc.h file and resolves the ECC values against the rest of the security
   header content.  Run the EccGen utility on the adi_ecc.h file to regenerate the
   ECC codes whenever the security header content is altered.

   FAILURE TO PROVIDE VALID ECC ENCODINGS RESULTS IN BOOT FAILURE.
*/

/* pick up security header defines, including encoded ECC values */
#include "adi_ecc.h"

/* appriopriate section ".intvec" for the security header
   and place the IVT at "ADI_SECURITY_IVT_ADDRESS"
*/
#pragma location = ".intvec"
__root const uint32_t SecurityHeader[] = {
    ADI_SECURITY_SIGNATURE0,     // sig0
    ADI_SECURITY_RESERVED0,      // res0
    ADI_SECURITY_USER_KEY0,      // key0
    ADI_SECURITY_USER_KEY1,      // key1
    ADI_SECURITY_USER_KEY2,      // key2
    ADI_SECURITY_USER_KEY3,      // key3
    ADI_SECURITY_BOOT_FLAGS,     // flags
    ADI_SECURITY_IVT_ADDRESS,    // ivt address
    ADI_SECURITY_ECC0,           // lower ECC field
    ADI_SECURITY_ECC1,           // upper ECC field
    ADI_SECURITY_SIGNATURE1,     // sig1
    ADI_SECURITY_SIGNATURE2      // sig2
};
#endif  /* not ADI_NO_SECURITY_PROTOCOL */


// IVT typedefs
typedef void  ( *IntFunc )( void );
typedef union { IntFunc __fun; void * __ptr; } IntVector;

/* import the CSTACK segment name from the linker description file */
#pragma segment="CSTACK"

// Locate IVT depending on security model
#ifndef ADI_NO_SECURITY_PROTOCOL
/* For SECURE PARTS: locate IVT at absolute address macro defined in adi_ecc.h security header */
#pragma location = ADI_SECURITY_IVT_ADDRESS
#else
/* For NON-SECURE PARTS: locate IVT at start of ".intvec" section */
#pragma location = ".intvec"
#endif /* not ADI_NO_SECURITY_PROTOCOL */

// Place the Interrupt Vector Table
__root const IntVector __vector_table[] =
{
    // grab stack pointer "end" address from ICF file
    { .__ptr = __sfe( "CSTACK" ) },

// exception mappings...
    {ResetISR                           },  // -15
};


#ifndef DO_NOT_RELOCATE_IVT
/*
    A relocated IVT is requested.  Provision for IVT relocation
    to RAM during startup.    This allows for dynamic interrupt
    vector patching required by RTOS.  Places the relocated IVT
    at the start of RAM.  Note: the IVT placement is required
    to be next power-of-two    of the vector table size.  So the
    IVT includes 61    programmable interrupts, 15 system exception
    vectors and    the main stack pointer, therefore we need
    (61 + 15 + 1)*4    = 308 bytes, which rounds up to a 512 (0x200)
    address    boundary (which address 0x20000000 satisfies).

    Also note use of the "__no_init" attibute to force the
    relocated IVT into the ".noinit" section.  This prevents
    the CRTL startup sequence from initializing the relocated
    IVT after we have activated it during the reset vector.
    so that the CRTL does not clear it.
*/
#define RELOCATION_ADDRESS   (0x10000000)
#define RELOCATION_ALIGNMENT      (0x200)
#define SIZEOF_IVT sizeof(__vector_table)
#if defined ( __ICCARM__ )
    #pragma data_alignment=RELOCATION_ALIGNMENT  /* IAR */
#elif defined (__CC_ARM)
    __align(RELOCATION_ALIGNMENT)                /* Keil */
#else
    #pragma message("WARNING: NO ALIGHMENT DEFINED FOR IVT RELOCATION")
#endif
/* reserve no-init aligned IVT space at top of RAM */
SECTION_PLACE(KEEP_VAR(__no_init IntVector __relocated_vector_table[SIZEOF_IVT/4]), RELOCATION_ADDRESS);

#endif //DO_NOT_RELOCATE_IVT


#ifdef __ARMCC_VERSION
// We want a warning if semi-hosting libraries are used.
#pragma import(__use_no_semihosting_swi)
#endif

//*****************************************************************************
//
// Application Entry point
//
//*****************************************************************************
extern int main(void);


//*****************************************************************************
// Symbols defined by the GNU linker.
//*****************************************************************************
#ifdef __GNUC__
extern unsigned long _etext;
extern unsigned long _data;
extern unsigned long _edata;
extern unsigned long _bss ;
extern unsigned long _ebss;
#endif

//*****************************************************************************
// IAR system startup functions.
//*****************************************************************************
#ifdef __ICCARM__
WEAK_PROTO( void __iar_init_core (void));
WEAK_PROTO( void __iar_init_vfp  (void));
extern      void __cmain();
#endif

//*****************************************************************************
// Function    : ResetISR (-15)
// Description : Reset event handler
//*****************************************************************************
WEAK_FUNC( void ResetISR (void)) {

    /*
        Note: the BOOT-ROM IVT is still active at this time, which sets
        the fault pin on any non-maskable exceptions.  After 1) the default
        IVT is copied to SRAM (below) and, 2) the interrupt handlers are
        copied (by the CRTL), we change the Interrupt Vector Table Offset
        Register (VTOR) in main as part of SystemInit().  Until this sequence
        is complete, the BOOT-ROM IVT remains active.
    */


    /* reset the main SP to clean up any leftovers from the boot rom. */
    __set_MSP( (uint32_t) __vector_table[0].__ptr );

    /* disable oscillator watchdog false positive anomoly */
    /* (avoid driving the fault light on false positives) */
    __disable_irq();
    pADI_SEC0->FCTL &= ~BITM_SEC_FCTL_EN;  //turn off fault out/in enables
    pADI_SEC0->FCTL &= ~(BITM_SEC_FCTL_FIEN | BITM_SEC_FCTL_FOEN);  //turn off fault out/in enables

    pADI_CGU0->OSCWDCTL &= ~BITM_CGU_OSCWDCTL_FAULTEN;
    __ISB();

    if (((pADI_SEC0->FSTAT & BITM_SEC_FSTAT_ACT) >> BITP_SEC_FSTAT_ACT) &
       ((pADI_SEC0->FEND & BITM_SEC_FSID_FEXT) >> BITP_SEC_FSID_FEXT)) {
       pADI_SEC0->FEND = BITM_SEC_FEND_FEXT;
       }

    pADI_SEC0->FCTL |= (BITM_SEC_FCTL_FOEN | BITM_SEC_FCTL_FIEN);
    pADI_SEC0->FCTL |= BITM_SEC_FCTL_EN;
    __enable_irq();

#ifndef DO_NOT_RELOCATE_IVT
    int i;
    uint8_t *pSrc, *pDst;

    /* copy the IVT (avoid use of memcpy here so it does not become locked into flash) */
    for (i=0, pSrc=(uint8_t*)__vector_table, pDst=(uint8_t*)__relocated_vector_table; i<SIZEOF_IVT; i++)
        *pDst++ = *pSrc++;

#endif //DO_NOT_RELOCATE_IVT

#ifdef __GNUC__

  unsigned long *pulSrc, *pulDest;

    // Copy initialised data from flash into RAM
    pulSrc = &_etext;
    for(pulDest = &_data; pulDest < &_edata; )
    {
        *pulDest++ = *pulSrc++;
    }

    // Clear the bss segment
    for(pulDest = &_bss; pulDest < &_ebss; )
    {
        *pulDest++ = 0;
    }
    // Call application main directly.
    main();

#elif __ICCARM__

    // Call IAR system startup
    __iar_init_core();
    __iar_init_vfp();
    __cmain();

#else

    // Call application main directly.
    main();

#endif

    // Stick here if main returns
    while(1);
}


//*****************************************************************************
// Provide default (weakly bound) trapping handlers to be overridden as needed
//*****************************************************************************
WEAK_FUNCTION( NmiSR                              )
WEAK_FUNCTION( HardFault_Handler                  )
WEAK_FUNCTION( MemManage_Handler                  )
WEAK_FUNCTION( BusFault_Handler                   )
WEAK_FUNCTION( UsageFault_Handler                 )
WEAK_FUNCTION( SVC_Handler                        )
WEAK_FUNCTION( DebugMon_Handler                   )
WEAK_FUNCTION( PendSV_Handler                     )
WEAK_FUNCTION( SysTick_Handler                    )

WEAK_FUNCTION( OSCW_EVT_Int_Handler               )
WEAK_FUNCTION( CGU0_EVT_Int_Handler               )
WEAK_FUNCTION( M4P_L1CC_PERR_Int_Handler          )
WEAK_FUNCTION( M4P_CORE_SRAM_PERR_Int_Handler     )
WEAK_FUNCTION( M4P_DMA_SRAM_PERR_Int_Handler      )
WEAK_FUNCTION( M4P_BUS_FAULT_Int_Handler          )
WEAK_FUNCTION( M4P_LOCKUP_Int_Handler             )
WEAK_FUNCTION( M4P_SRAM_PERR_FLT_Int_Handler      )
WEAK_FUNCTION( WDOG0_EXP_Int_Handler              )
WEAK_FUNCTION( SEC0_ERR_Int_Handler               )
WEAK_FUNCTION( ECT_EVT0_Int_Handler               )
WEAK_FUNCTION( ECT_EVT1_Int_Handler               )
WEAK_FUNCTION( PWM0_TRIP_Int_Handler              )
WEAK_FUNCTION( PWM1_TRIP_Int_Handler              )
WEAK_FUNCTION( PWM2_TRIP_Int_Handler              )
WEAK_FUNCTION( PWM0_SYNC_Int_Handler              )
WEAK_FUNCTION( PWM1_SYNC_Int_Handler              )
WEAK_FUNCTION( PWM2_SYNC_Int_Handler              )
WEAK_FUNCTION( PINT0_BLOCK_Int_Handler            )
WEAK_FUNCTION( PINT1_BLOCK_Int_Handler            )
WEAK_FUNCTION( PINT2_BLOCK_Int_Handler            )
WEAK_FUNCTION( PINT3_BLOCK_Int_Handler            )
WEAK_FUNCTION( PINT4_BLOCK_Int_Handler            )
WEAK_FUNCTION( SPORT0_A_DMA_ERR_Int_Handler       )
WEAK_FUNCTION( SPORT0_B_DMA_ERR_Int_Handler       )
WEAK_FUNCTION( SPI0_TXDMA_ERR_Int_Handler         )
WEAK_FUNCTION( SPI0_RXDMA_ERR_Int_Handler         )
WEAK_FUNCTION( UART0_TXDMA_ERR_Int_Handler        )
WEAK_FUNCTION( UART0_RXDMA_ERR_Int_Handler        )
WEAK_FUNCTION( SPORT1_A_DMA_ERR_Int_Handler       )
WEAK_FUNCTION( SPORT1_B_DMA_ERR_Int_Handler       )
WEAK_FUNCTION( SPI1_TXDMA_ERR_Int_Handler         )
WEAK_FUNCTION( SPI1_RXDMA_ERR_Int_Handler         )
WEAK_FUNCTION( UART1_TXDMA_ERR_Int_Handler        )
WEAK_FUNCTION( UART1_RXDMA_ERR_Int_Handler        )
WEAK_FUNCTION( UART2_TXDMA_ERR_Int_Handler        )
WEAK_FUNCTION( UART2_RXDMA_ERR_Int_Handler        )
WEAK_FUNCTION( HAE0_RXDMA_CH0_ERR_Int_Handler     )
WEAK_FUNCTION( HAE0_RXDMA_CH1_ERR_Int_Handler     )
WEAK_FUNCTION( HAE0_TXDMA_ERR_Int_Handler         )
WEAK_FUNCTION( SPI0_ERR_Int_Handler               )
WEAK_FUNCTION( SPI1_ERR_Int_Handler               )
WEAK_FUNCTION( SPI2_ERR_Int_Handler               )
WEAK_FUNCTION( ADCC0_ERR_Int_Handler              )
WEAK_FUNCTION( DACC0_ERR_Int_Handler              )
WEAK_FUNCTION( MDMA0_SRC_CRC0_IN_ERR_Int_Handler  )
WEAK_FUNCTION( MDMA0_DST_CRC0_OUT_ERR_Int_Handler )
WEAK_FUNCTION( MDMA1_SRC_ERR_Int_Handler          )
WEAK_FUNCTION( MDMA1_DST_ERR_Int_Handler          )
WEAK_FUNCTION( TIMER0_STAT_Int_Handler            )
WEAK_FUNCTION( TIMER0_TMR0_Int_Handler            )
WEAK_FUNCTION( TIMER0_TMR1_Int_Handler            )
WEAK_FUNCTION( TIMER0_TMR2_Int_Handler            )
WEAK_FUNCTION( TIMER0_TMR3_Int_Handler            )
WEAK_FUNCTION( TIMER0_TMR4_Int_Handler            )
WEAK_FUNCTION( TIMER0_TMR5_Int_Handler            )
WEAK_FUNCTION( TIMER0_TMR6_Int_Handler            )
WEAK_FUNCTION( TIMER0_TMR7_Int_Handler            )
WEAK_FUNCTION( CNT0_STAT_Int_Handler              )
WEAK_FUNCTION( CNT1_STAT_Int_Handler              )
WEAK_FUNCTION( CNT2_STAT_Int_Handler              )
WEAK_FUNCTION( CNT3_STAT_Int_Handler              )
WEAK_FUNCTION( SPORT0_A_STAT_Int_Handler          )
WEAK_FUNCTION( SPORT0_A_DMA_Int_Handler           )
WEAK_FUNCTION( SPORT0_B_STAT_Int_Handler          )
WEAK_FUNCTION( SPORT0_B_DMA_Int_Handler           )
WEAK_FUNCTION( UART0_STAT_Int_Handler             )
WEAK_FUNCTION( UART0_TXDMA_Int_Handler            )
WEAK_FUNCTION( UART0_RXDMA_Int_Handler            )
WEAK_FUNCTION( SINC0_STAT_Int_Handler             )
WEAK_FUNCTION( SPI0_STAT_Int_Handler              )
WEAK_FUNCTION( SPI0_TXDMA_Int_Handler             )
WEAK_FUNCTION( SPI0_RXDMA_Int_Handler             )
WEAK_FUNCTION( SPORT1_A_STAT_Int_Handler          )
WEAK_FUNCTION( SPORT1_A_DMA_Int_Handler           )
WEAK_FUNCTION( SPORT1_B_STAT_Int_Handler          )
WEAK_FUNCTION( SPORT1_B_DMA_Int_Handler           )
WEAK_FUNCTION( UART1_STAT_Int_Handler             )
WEAK_FUNCTION( UART1_TXDMA_Int_Handler            )
WEAK_FUNCTION( UART1_RXDMA_Int_Handler            )
WEAK_FUNCTION( SPI1_STAT_Int_Handler              )
WEAK_FUNCTION( SPI1_TXDMA_Int_Handler             )
WEAK_FUNCTION( SPI1_RXDMA_Int_Handler             )
WEAK_FUNCTION( SPI2_TX_Int_Handler                )
WEAK_FUNCTION( SPI2_RX_Int_Handler                )
WEAK_FUNCTION( EMAC0_STAT_Int_Handler             )
WEAK_FUNCTION( UART2_STAT_Int_Handler             )
WEAK_FUNCTION( UART2_TXDMA_Int_Handler            )
WEAK_FUNCTION( UART2_RXDMA_Int_Handler            )
WEAK_FUNCTION( HAE0_STAT_Int_Handler              )
WEAK_FUNCTION( HAE0_RXDMA_CH0_Int_Handler         )
WEAK_FUNCTION( HAE0_RXDMA_CH1_Int_Handler         )
WEAK_FUNCTION( HAE0_TXDMA_Int_Handler             )
WEAK_FUNCTION( ADCC0_TMR0_EVT_Int_Handler         )
WEAK_FUNCTION( ADCC0_TMR1_EVT_Int_Handler         )
WEAK_FUNCTION( DACC0_DAC0_Int_Handler             )
WEAK_FUNCTION( DACC0_DAC1_Int_Handler             )
WEAK_FUNCTION( SPI2_STAT_Int_Handler              )
WEAK_FUNCTION( TWI0_DATA_Int_Handler              )
WEAK_FUNCTION( CRC0_DCNTEXP_Int_Handler           )
WEAK_FUNCTION( CRC0_ERR_Int_Handler               )
WEAK_FUNCTION( MDMA0_SRC_CRC0_IN_Int_Handler      )
WEAK_FUNCTION( MDMA0_DST_CRC0_OUT_Int_Handler     )
WEAK_FUNCTION( MDMA1_SRC_Int_Handler              )
WEAK_FUNCTION( MDMA1_DST_Int_Handler              )
WEAK_FUNCTION( USB0_STAT_Int_Handler              )
WEAK_FUNCTION( USB0_DATA_Int_Handler              )
WEAK_FUNCTION( TRU0_INT0_Int_Handler              )
WEAK_FUNCTION( TRU0_INT1_Int_Handler              )
WEAK_FUNCTION( TRU0_INT2_Int_Handler              )
WEAK_FUNCTION( TRU0_INT3_Int_Handler              )
WEAK_FUNCTION( CGU0_ERR_Int_Handler               )
WEAK_FUNCTION( DPM0_EVT_Int_Handler               )
WEAK_FUNCTION( SOFT0_Int_Handler                  )
WEAK_FUNCTION( SOFT1_Int_Handler                  )
WEAK_FUNCTION( SOFT2_Int_Handler                  )
WEAK_FUNCTION( SOFT3_Int_Handler                  )
WEAK_FUNCTION( SWU0_EVT_Int_Handler               )
WEAK_FUNCTION( SWU1_EVT_Int_Handler               )
WEAK_FUNCTION( SWU2_EVT_Int_Handler               )
WEAK_FUNCTION( SWU3_EVT_Int_Handler               )
WEAK_FUNCTION( SWU4_EVT_Int_Handler               )
WEAK_FUNCTION( CAN0_RX_Int_Handler                )
WEAK_FUNCTION( CAN0_TX_Int_Handler                )
WEAK_FUNCTION( CAN0_STAT_Int_Handler              )
WEAK_FUNCTION( CAN1_RX_Int_Handler                )
WEAK_FUNCTION( CAN1_TX_Int_Handler                )
WEAK_FUNCTION( CAN1_STAT_Int_Handler              )
WEAK_FUNCTION( CPTMR0_CPT0_ERR_Int_Handler        )
WEAK_FUNCTION( CPTMR0_CPT1_ERR_Int_Handler        )
WEAK_FUNCTION( CPTMR0_CPT2_ERR_Int_Handler        )
WEAK_FUNCTION( CPTMR0_CPT0_MEAS_Int_Handler       )
WEAK_FUNCTION( CPTMR0_CPT1_MEAS_Int_Handler       )
WEAK_FUNCTION( CPTMR0_CPT2_MEAS_Int_Handler       )
WEAK_FUNCTION( USER_KEY_FAIL_Int_Handler          )
WEAK_FUNCTION( RESERVED_VECTOR                    )
